/*************************************************************************

  list.h v2.1

    To maintain a linked list of dynamically allocated items.
    list.h is part of the Oankali GP32 run-time library.

  Copyright (c) 2004,2005 by
  Guillem Reinoso <oankali@nekanium.com>
  www.nekanium.com

  These software is freeware and is provided as is.
  If it doesn't work for you or it makes your PC or GP32 melt down, it's not my
  responsibility.
  If you use it in your programs, cite me please.

  History:
    v1.0
      First public release
    v2.0
      Added     ListMakeFirst(), ListMakeLast() and ListSwap().
      Changed   ListAddItem() -> if pObject is not NULL and size > 0, the object
                data is copied automatically in the new instanciated item object.
      Changed   ListDestroy() -> items in the list are now destroy from the end.
      Changed   ListGetItemByObject() -> the item is now searched from the end.
    v2.1
      Added     ListClear().
      Changed   ListDestroy() -> now uses ListClear().

**************************************************************************/


#ifndef __list_h__
#define __list_h__

#ifdef __cplusplus
extern "C" {
#endif


// Structures
//- An item of the list
typedef struct tagLISTITEM {
  struct tagLIST     *pList;       // a pointer to the header of the list
  struct tagLISTITEM *pPrevious;   // a pointer to the previous item in the list
  struct tagLISTITEM *pNext;       // a pointer to the next item in the list

  void *pObject;                   // a pointer to the object data
  unsigned char autoAllocated;     // TRUE if object has been created automatically
  unsigned short handle;           // unique handle of the item
} LISTITEM;

//- A list of objects
typedef struct tagLIST {
  LISTITEM *pFirst;                // a pointer to the first item in the list
  LISTITEM *pLast;                 // a pointer to the last item in the list
  unsigned short count;            // number of items in the list
  unsigned short lastHandle;       // last given unique item handle
} LIST;


// Prototypes
LISTITEM *ListAddItem(LIST *pList, void *pObject, unsigned long size);
void      ListClear(LIST *pList);
LIST     *ListCreate(void);
void      ListDestroy(LIST *pList);
int       ListGetIndex(LISTITEM *pItem);
int       ListGetIndexByHandle(LIST *pList, int handle);
LISTITEM *ListGetItemByHandle(LIST *pList, int handle);
LISTITEM *ListGetItemByIndex(LIST *pList, int index);
LISTITEM *ListGetItemByObject(LIST *pList, void *pObject);
void     *ListGetObjectByHandle(LIST *pList, int handle);
void     *ListGetObjectByIndex(LIST *pList, int index);
LISTITEM *ListGetRelativeItem(LISTITEM *pItem, int relIndex);
int       ListGetRelativePosition(LISTITEM *pItem1, LISTITEM *pItem2);
void      ListMakeFirst(LISTITEM *pItem);
void      ListMakeLast(LISTITEM *pItem);
void      ListRemoveItem(LISTITEM *pItem);
void      ListRemoveItemByHandle(LIST *pList, int handle);
void      ListRemoveItemByIndex(LIST *pList, int index);
void      ListRemoveItemByObject(LIST *pList, void *pObject);
void      ListSort(LIST *pList, int (*pCompare)(LISTITEM *pItem1, LISTITEM *pItem2));
void      ListSwap(LISTITEM *pItem1, LISTITEM *pItem2);


#ifdef __cplusplus
  }
#endif

#endif /*__list_h__*/
