/*************************************************************************

  riff.h v1.0

    To manage RIFF files.
    riff.h is part of the Oankali's GP32 run-time library.

  Copyright (c) 2005 by
  Guillem Reinoso <oankali@nekanium.com>
  www.nekanium.com

  These software is freeware and is provided as is.
  If it doesn't work for you or it makes your PC or GP32 melt down, it's not my
  responsibility.
  If you use it in your programs, cite me please.

**************************************************************************/


#ifndef __riff_h__
#define __riff_h__

#ifdef __cplusplus
extern "C" {
#endif

#include <gpstdio.h>

// RIFF files type definitions
typedef unsigned long DWORD; 
typedef unsigned char BYTE; 
typedef DWORD FOURCC;         // Four-character code 

// RIFF error codes
typedef enum {
  riffOK,                // Everything's fine!
  riffError,             // Unknown error
  riffFileNotFound,      // RIFF file not found
  riffNotRIFFFile,       // File is not a RIFF file
  riffCorruptedRIFFFile, // RIFF file is corrupted
  riffInvalidFormType,   // Form type is invalid
  riffChunkNotFound,     // Chunk name not found
  riffReadError,         // Read error
  riffWriteError         // Write error
} RIFF_ErrCode;


// Macro definitions
#define MAKEFOURCC(ch0, ch1, ch2, ch3)  \
              (((DWORD)(char)(ch0))       | ((DWORD)(BYTE)(ch1) << 8) |  \
               ((DWORD)(char)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24 ))
    

// Prototypes
RIFF_ErrCode RiffClose (F_HANDLE hFile);
RIFF_ErrCode RiffCreate (char *szFilename, FOURCC fccFormType, F_HANDLE *hFile);
RIFF_ErrCode RiffGetChunkData (F_HANDLE hFile, FOURCC fccChunkId, void *vBuffer, DWORD lBufferSize);
RIFF_ErrCode RiffGetChunkSize (F_HANDLE hFile, FOURCC fccChunkId, DWORD *lSize);
RIFF_ErrCode RiffGetFileChunkData (char *szFilename, FOURCC fccFormType, FOURCC fccChunkId, void *vBuffer, DWORD lBufferSize);
RIFF_ErrCode RiffGetFormType (F_HANDLE hFile, FOURCC *fccFormType);
RIFF_ErrCode RiffSaveChunkData (F_HANDLE hFile, FOURCC fccChunkId, void *vBuffer, DWORD lBufferSize);
RIFF_ErrCode RiffSaveFileChunkData (char *szFilename, FOURCC fccFormType, FOURCC fccChunkId, void *vBuffer, DWORD lBufferSize);
RIFF_ErrCode RiffOpen (char *szFilename, F_HANDLE *hFile);


#ifdef __cplusplus
  }
#endif

#endif /*__riff_h__*/
